<?php

/**
  Copyright (c) 2016 VASYLTECH.COM

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
 */

/**
 * Error Fix connector
 * 
 * @package AAM
 * @author Vasyl Martyniuk <vasyl@vasyltech.com>
 */
class ErrorFix_Connect {
    
    /**
     * 
     */
    public function __construct() {
        $id     = filter_input(INPUT_GET, 'id');
        $action = filter_input(INPUT_GET, 'action');
        
        //verify request
        if ($id == ErrorFix_Core_Option::getId()) {
            //clear the output buffer
            while (@ob_end_clean()) {}
            
            if (method_exists($this, $action)) {
                call_user_func(array($this, $action));
                exit;
            }
        }
    }
    
    /**
     * Discover the website
     * 
     * Simply check if there is a successful connection with this website
     * 
     * @return void
     * 
     * @access protected
     */
    protected function discover() {
        if (filter_input(INPUT_GET, 'info')) {
            ob_start();
            phpinfo(INFO_GENERAL | INFO_CONFIGURATION | INFO_MODULES);
            $info = ob_get_contents();
            ob_end_clean();
        } else {
            $info = null;
        }
        
        echo json_encode(array(
            'site'    => site_url(),
            'phpinfo' => $info 
        ));
    }
    
    /**
     * Fetch website data
     * 
     * Remotely get the website PHP error log, individual file or whole module
     * 
     * @return void
     * 
     * @access protected
     */
    protected function download() {
        switch(filter_input(INPUT_GET, 'object')) {
            case 'file':
                $this->fetchFile();
                break;
            
            case 'module':
                $this->fetchModule();
                break;
            
            default:
                break;
        }
    }
    
    /**
     * Fetch individual file
     * 
     * @return void
     * 
     * @access protected
     * @since 3.3.6
     */
    protected function fetchFile() {
        $error = $this->getErrorByReport();
        
        if (!empty($error)) {
            $zip      = new PclZip(ErrorFix_Core::get('basedir') . '/' . uniqid());
            $filepath = str_replace('\\', '/', $error->filepath);
            
            $zip->create(
                $error->filepath, 
                ErrorFix_Core::get('instance'),
                str_replace($error->relpath, '', $filepath)
            );
            
            if ($zip->errorCode() == PCLZIP_ERR_NO_ERROR) {
                $this->forceDownload(basename($zip->zipname));
                echo file_get_contents($zip->zipname);
                unlink($zip->zipname);
            } else {
                echo $zip->errorInfo(true);
            }
        } else {
            header('HTTP/1.1 404 Not Found');
        }
    }
    
    /**
     * Fetch module
     * 
     * @return void
     * 
     * @access protected
     */
    protected function fetchModule() {
        $error = $this->getErrorByReport();
        
        if (!empty($error)) {
            $zip = new PclZip(ErrorFix_Core::get('basedir') . '/' . uniqid());
            
            $zip->create(
                $error->module['path'], 
                PCLZIP_OPT_REMOVE_PATH, 
                dirname($error->module['path'])
            );
            
            if ($zip->errorCode() == PCLZIP_ERR_NO_ERROR) {
                $this->forceDownload(basename($error->module['path']));
                echo file_get_contents($zip->zipname);
                unlink($zip->zipname);
            } else {
                echo $zip->errorInfo(true);
            }
        } else {
            header('HTTP/1.1 404 Not Found');
        }
    }
    
    /**
     * 
     */
    protected function forceDownload($filename) {
        if (filter_input(INPUT_GET, 'inbrowser')) {
            header("Content-Type: application/zip");
            header("Content-Transfer-Encoding: Binary");
            header("Content-disposition: attachment; filename=\"{$filename}.zip\"");
        }
    }
    
    /**
     * 
     * @return type
     */
    protected function getErrorByReport() {
        $report = filter_input(INPUT_GET, 'report');
        
        //find the error by the report
        $error = null;
        foreach(ErrorFix_Storage::getInstance()->getErrors() as $item) {
            if ($item->report == $report) {
                $error = $item;
                break;
            }
        }
        
        return $error;
    }
    
}